# -*- coding: utf-8 -*-

__author__ = 'luckydonald'

from random import randint  # needed for eastereggs :D

from .. import py3
from ..logger import logging
from ..encoding import to_binary
from ..eastereggs.headers import get_headers
logger = logging.getLogger(__name__)


try:
	if py3:
		#from socketserver import TCPServer
		from http.server import SimpleHTTPRequestHandler, HTTPServer
		from urllib.parse import parse_qs
	else:
		#from SocketServer import TCPServer
		from SimpleHTTPServer import SimpleHTTPRequestHandler
		from BaseHTTPServer import HTTPServer
		from urlparse import parse_qs
	#end if
except ImportError:
	raise  # try-except is only needed to satisfy PyCharm; to avoid PyCharm marking that as wrong import. As consequence the old error is reraised, as it really is an error.
#end try
from cgi import parse_header, parse_multipart
from socket import error  # retry init'ing
from errno import EADDRINUSE  # retry init'ing
from time import sleep  # retry init'ing
from os import path


class BetterHTTPRequestHandler(SimpleHTTPRequestHandler, object):
	def __init__(self, request, client_address, server):
		if py3:
			super(BetterHTTPRequestHandler, self).__init__(request, client_address, server)
		else:
			SimpleHTTPRequestHandler.__init__(self, request, client_address, server)
		self.static_files_dir = self.server.static_files_dir[:]  # force a copy

	def do_GET(self):
		logger.warn("do_GET is not overridden!")
		if self.path == "/":
			self.path = "/index.html"
		if False:  # Copy this and add your code here. ```if self.path == "/foo.bar":```
			pass
		else:
			self.path = self.static_files_dir + self.path  # e.g. localhost/123/foo.bar -> /path/to/script/webserver_files/123/foo.bar
			logger.debug("Requested file {file}".format(file=self.path))
			f = self.send_head()  # this handles 404'ing for us
			if f:
				try:
					self.copyfile(f, self.wfile)
				finally:
					f.close()
				# end try
			# end if
		#end if-elif-else
	# end def

	def do_POST(self):
		logger.warn("do_POST is not overridden!")
		if self.path == "/":
			self.path = "/index.html"
		self.post_data = self.parse_POST()
		if False:  # Copy this and add your code here. ```if self.path == "/foo.bar":```
			pass
		else:
			self.path = self.static_files_dir + self.path  # e.g. localhost/123/foo.bar -> /path/to/script/webserver_files/123/foo.bar
			logger.debug("Requested file {file}".format(file=self.path))
			f = self.send_head()  # this handles 404'ing for us
			if f:
				try:
					self.copyfile(f, self.wfile)
				finally:
					f.close()
				# end try
			# end if
		#end if-elif-else
	#end def


	def parse_POST(self):
		ctype, pdict = parse_header(self.headers['content-type'])
		if "boundary" in pdict:
			pdict["boundary"] = to_binary(pdict["boundary"])
		if ctype == 'multipart/form-data':
			postvars = parse_multipart(self.rfile, pdict)
		elif ctype == 'application/x-www-form-urlencoded':
			length = int(self.headers['content-length'])
			postvars = parse_qs(
					self.rfile.read(length),
					keep_blank_values=1)
		else:
			postvars = {}
		return postvars
	#end def

	def write_text(self, msg, content_type="text/plain", is_binary=False):
		"""
		Makes answering with text/data easier.

		:param msg: The text to send to the browser/client.
		:keyword content_type: If you don't like it to be text, change that here.
		:keyword is_binary: Text (unicode) needs to be converted to binary. If you already have binary (e.g. an PNG as binary data) you can set that here.
		:return: Nothing.
		"""
		# Now do servery stuff.
		if not is_binary:
			msg = to_binary(msg)
		if msg is None:
			self.send_response(404)
			self.end_headers()
			return
		self.send_response(200)
		self.send_header("Content-type", content_type)
		self.send_header("Content-Length", str(len(msg)))
		for k, v in get_headers().items():
			self.send_header(k, v)
		self.send_header("X-Backend-created-by", "luckydonald")
		self.send_header("X-Licence", "Luna-Will-Cry-If-You-Modify-Or-Redistribute 1.0 or later")
		self.send_header("X-Licence-URL", "flutterb.at/lwc-1-0")
		self.end_headers()
		self.wfile.write(msg)
		return
	# end def

	def translate_path(self, path_):
		"""
		 Now accepts local fitting paths automatically
		E.g. "/path/to/www-dir/foo.png" is valid if that folder exists.
		Now it won't change the path to "/path/to/www-dir/foo.png/path/to/www-dir/foo.png", like it did before.
		:param path: path for the webserver.
		:return:
		"""
		if path.exists(path_):
			return path_
		if py3:
			super(BetterHTTPRequestHandler, self).translate_path(path_)
		else:
			SimpleHTTPRequestHandler.translate_path(self, path_)
		return path_
	# end def

	def log_message(self, format, *args):
		logger.info("%s - %s - [%s] %s\n" % (self.client_address[0], self.path,self.log_date_time_string(), format % args))

	def log_request(self, code='-', size='-'):
		logger.debug('%s - %s - "%s" %s %s', self.client_address[0],self.path, self.requestline, str(code), str(size))

	def log_error(self, format, *args):
		logger.error("%s - %s - %s" % (self.client_address[0], self.path, format % args))
#end class


def start_a_webserver(handler, port, host="", static_files_dir=None):
	"""
	Starts a ```TCPServer```, using the given ```handler```,
	:param handler: An handler instance, e.g. an ```BetterHTTPRequestHandler```
	:param port: The port where to serve on. For example ```80``` or ```8080``` for HTTP (```80``` often needs root privileges)
	:keyword host: Optional. A host where to serve on. If an empty string ```""``` (default) is given, all incoming connections are allowed. (from localhost, from lan, from internet, etc.)
	:keyword static_files_dir: Optional. Set the ```static_files_dir``` of an ```BetterHTTPRequestHandler``` handler.
	:return: The ```TCPServer``` created.
	"""
	#assert (isinstance(handler, BetterHTTPRequestHandler))  # BaseRequestHandler?
	#handler = BetterHTTPRequestHandler()
	httpd = None
	started = False
	while not started:
		try:
			httpd = HTTPServer((host, port), handler)
			if issubclass(handler, BetterHTTPRequestHandler):
				if not static_files_dir:
					logger.warn("`static_files_dir` kwarg-parameter should be used when a `BetterHTTPRequestHandler` is used.")
				httpd.static_files_dir = static_files_dir
			started = True
		except error as e:
			if e.errno in [EADDRINUSE]:
				logger.warn("Starting Server failed. Address already in use. Retrying.")
				sleep(1)
			else:
				raise
			#end if-else
		#end try
	#end while
	return httpd
#end def



# -*- coding: utf-8 -*-
__author__ = 'luckydonald'


from collections import defaultdict

def etree_to_dict(t):
	"""
	Function to modify a xml.etree.ElementTree thingy to be a dict.
	Attributes will be accessible via ["@attribute"],
	and get the text (aka. content) inside via ["#text"]

	TESTED ONLY FOR PYTHON 3! (but should be working in Python 2...)
	:param t:
	:return:
	"""
	# THANKS http://stackoverflow.com/a/10077069

	d = {t.tag: {} if t.attrib else None}
	children = list(t)
	if children:
		dd = defaultdict(list)
		for dc in map(etree_to_dict, children):
			for k, v in dc.items():
				dd[k].append(v)
		d = {t.tag: {k:v[0] if len(v) == 1 else v for k, v in dd.items()}}  # .items() is bad for python 2
	if t.attrib:
		d[t.tag].update(('@' + k, v) for k, v in t.attrib.items())  # .items() is bad for python 2
	if t.text:
		text = t.text.strip()
		if children or t.attrib:
			if text:
				d[t.tag]['#text'] = text
		else:
			d[t.tag] = text
	return d


# -*- coding: utf-8 -*-
__author__ = 'luckydonald'


from django.conf import settings
from django.core.urlresolvers import reverse, NoReverseMatch
from django.http import HttpResponseForbidden
from ...logger import logging  # pip install luckydonald-utils
from ...dependencies import import_or_install
logger = logging.getLogger(__name__)

import_or_install("IPy", "IPy")
from IPy import IP  # pip install IPy


class AllowFromIPMiddleware(object):
	"""
	Allow only given IPs to access, else raise a `Http404` error. Is ignoren when `DEBUG` is `True`.
	Checks ranges with https://pypi.python.org/pypi/IPy/ .

	Example:
	The *`settings.py`*
	```
	ALLOW_FROM = ["8.8.8.8", "134.169.0.0/16"]
	```
	Empty array means allow 'all'.

	Mimics the Apache syntax:
	> Allow from 134.169.
	One would write as
	>>> ALLOW_FROM = ["134.169"]
	or
	>>> ALLOW_FROM = ["134.169.0.0/16"]
	in the `settings.py`.
	"""
	buffered_ALLOW_FROM = None

	def process_request(self, request):
		if self.buffered_ALLOW_FROM is None:
			self.buffered_ALLOW_FROM = []
			for ip in settings.ALLOW_FROM:
				self.buffered_ALLOW_FROM.append(IP(ip))
			#end for
		#end if
		if len(self.buffered_ALLOW_FROM) == 0:
			return  # allow
		remote_addr = request.META.get('HTTP_X_REAL_IP', request.META.get('REMOTE_ADDR', None))
		if any([remote_addr in ip for ip in self.buffered_ALLOW_FROM]):
			return  # allow
		if settings.DEBUG:
			logger.warn("The IP {remote_addr} should not be allowed, but is served anyway because DEBUG=True.".format(remote_addr=remote_addr))
			return  # allow
		return HttpResponseForbidden("Your ip {ip} is not allowed.".format(ip=remote_addr))   # forbid
			#end if DEBUG
		#end if
	#end def
#end class


# -*- coding: utf-8 -*-
__author__ = 'luckydonald'

""" Python 3 only """

def success(self, msg, *args, exc_info=False, **kwargs):
	"""
	Log 'msg % args' with severity 'SUCCESS'.

	To pass exception information, use the keyword argument exc_info with
	a true value.

	logger.debug("Houston, we landed in the %s", "moon", exc_info=False)
	"""
	kwargs["exc_info"] = exc_info
	self._success(msg, *args, **kwargs)


# -*- coding: utf-8 -*-

__author__ = 'luckydonald'

__all__ = ["logging", "ColoredFormatter"]

import logging as _logging
import sys

class ColoredFormatter(_logging.Formatter):
	class Color(object):
		"""
		 utility to return ansi colored text.
		 just to store the colors next to the function.
		"""
		# Color codes: http://misc.flogisoft.com/bash/tip_colors_and_formatting

		colors = {
			'default': 39,
			'black': 30,
			'red': 31,
			'green': 32,
			'yellow': 33,
			'blue': 34,
			'magenta': 35,
			'cyan': 36,
			'white': 37,
			'grey': 90,
			'bgred': 41,
			'bggrey': 100
		}
		mapping = {
			'INFO': 'default',
			'WARNING': 'yellow',
			'ERROR': 'red',
			'CRITICAL': 'magenta',
			'DEBUG': 'grey',
			'SUCCESS': 'green'
		}

		color_prefix = '\033['

		def prepare_color(self, color_number):
			return ('%s%dm') % (self.color_prefix, color_number)
		# end def

		def colored(self, record):
			"""
			adsasd
			"""

			color = self.mapping.get(record.levelname, 'default')
			clr   = self.colors[color]

			filepart = record.treadName + ": " if hasattr(record, "treadName") and record.treadName != "MainTread" else ""
			filepart += record.name if record.name else ""
			filepart += "." + record.funcName + ": " if record.funcName != "<module>" else ": "
			filepart = filepart.join([self.prepare_color(94), self.prepare_color(39)]) #  Light blue ,  Default foreground color
			formatter = dict(
				all_off=self.prepare_color(0),		  # Reset all attributes
				color_on=self.prepare_color(clr),     #  Color as given/from lookup
				color_off=self.prepare_color(39),     #   Default foreground color
				inverse_on=self.prepare_color(7),     #    Reverse (invert the foreground and background colors)
				inverse_off=self.prepare_color(27),   #     Reset reverse
				background_off=self.prepare_color(49),#      Default background color)
			)
			lines = []
			level = "{:8}".format(record.levelname)
			level_filler = "{:{}}".format("",len(level))
			lines_ = record.message.splitlines()
			first_line = True  if len(lines_) > 1 else  None
			for line in lines_:
				if first_line is None: # single line
					lines.append("{color_on}{inverse_on}{level}{inverse_off}{color_off} {filepart}{color_on}{message}{color_off}{background_off}{all_off}".format(filepart=filepart, level=level,message=line, **formatter))
					break
				elif first_line: # first line
					lines.append("{color_on}{inverse_on}{level}{inverse_off}{color_off} {filepart}{all_off}".format(filepart=filepart, level=level,message=line, **formatter))
				lines.append("{color_on}{inverse_on}{level_filler}{inverse_off}{color_off} {color_on}{message}{color_off}{background_off}{all_off}".format(level_filler=level_filler,message=line, **formatter))
				first_line = False
			# end for
			return "\n".join(lines)
		# end def

	colored = Color().colored
	#firstpart = _logging.Formatter("%(levelname)s: %(threadName)s %(name)s.%(funcName)s: %(message)s")
	def format(self, record):
		super(ColoredFormatter, self).format(record)
		#if record.threadName == "MainThread":
		#	pass
		#part1 = self.firstpart.format(record)
		s = self._fmt % record.__dict__  # py3: s = self.formatMessage(record)
		if record.exc_text:
			if s[-1:] != "\n":
				s += "\n"
			try:
				s = s + record.exc_text
			except UnicodeError:  # PYTHON 2, LOL!
				# Sometimes filenames have non-ASCII chars, which can lead
				# to errors when s is Unicode and record.exc_text is str
				# See issue 8924.
				# We also use replace for when there are multiple
				# encodings, e.g. UTF-8 for the filesystem and latin-1
				# for a script. See issue 13232.
				s = s + record.exc_text.decode(sys.getfilesystemencoding(), 'replace')
		if hasattr(record, "stack_info") and record.stack_info:  # py2 doesn't have .stack_info
			if s[-1:] != "\n":
				s += "\n"
			s = s + record.stack_info  # py3: self.formatStack()
		record.message = s
		return self.colored(record)


# noinspection PyProtectedMember,PyProtectedMember
class _LoggingWrapper(object):
	SUCCESS = 25 # between WARNING and INFO
	def __init__(self):
		_logging.addLevelName(self.SUCCESS, 'SUCCESS')

	def getLoglevelInt(self, level_string):
		"""
		You provide a String, and get a level int
		:param level_string: The level.
		:type  level_stirng: str
		:return: level
		:rtype : int
		:raises KeyError: if the level does not exists.
		"""
		return {
			"NOTSET":_logging.NOTSET,
			"DEBUG":_logging.DEBUG,
			"INFO":_logging.INFO,
			"SUCCESS":self.SUCCESS,
			"WARNING":_logging.WARNING,
			"WARN":_logging.WARN, # = WARNING
			"ERROR":_logging.ERROR,
			"FATAL":_logging.FATAL, # = CRITICAL
			"CRITICAL":_logging.CRITICAL,
			} [level_string]
	def __call__(self, logger_name):
		"""
		alias to logger.getLogger(logger_name)
		:param logger_name:
		:return: self.getLogger(logger_name)
		"""
		return self.getLogger(logger_name)

	def add_colored_handler(self, logger_name=None, stream=None, level=None):
		"""
		Register a logger handler to colorfull print the messages.

		If stream is specified, the instance will use it for logging output; otherwise, sys.stdout will be used.

		:param logger_name: the name of the logger you want to register the printing to.
		Probably you should use __name__ , to use your package's logger, "root" will force all loggers to output. DO NOT use this in librarys.

		:param stream: An output stream. Default: sys.stdout

		:return: None
		"""
		logger = self.getLogger(logger_name)  # __name__
		if stream is None:
			import sys
			stream = sys.stdout
		handler = _logging.StreamHandler(stream=stream)
		formatter = ColoredFormatter()
		handler.setFormatter(formatter)
		logger.addHandler(handler)
		if level:
			logger.setLevel(level)
		return logger

	def test_logger_levels(self):
		logger = self.getLogger(__name__)
		logger_level = logger.getEffectiveLevel()
		logger.setLevel(logging.DEBUG)
		logger.debug('level debug')
		logger.info('level info')
		logger.success('level success')
		logger.warning('level warning')
		logger.error('level error')
		logger.critical('level critical')
		logger.setLevel(logger_level)

	def getLogger(self, name=None):
		"""
		Adds the .success() function to the logger, else it is same as logger.getLogger()
		:param logger: a logging.getLogger() logger.
		:return:
		"""
		logger = _logging.getLogger(name)
		logger.SUCCESS = self.SUCCESS
		setattr(logger, "success", lambda message, *args: logger._log(self.SUCCESS, message, args))
		return logger

	if sys.version < "3":
		def success(self, msg, *args, **kwargs):
			"""
			Log 'msg % args' with severity 'SUCCESS'.

			To pass exception information, use the keyword argument exc_info with
			a true value.

			logger.debug("Houston, we landed in the %s", "moon", exc_info=False)
			"""
			self._success(msg, *args, **kwargs)
	else:
		from .py3 import success

	def _success(self, msg, *args, **kwargs):
		if len(self.root.handlers) == 0:
			self.basicConfig()
		self.root._log(self.SUCCESS, msg, args, **kwargs)

	def __getattr__(self, item):
		if item != "__getattr__":
			if item in self.__dict__:
				return self.__dict__[item]
		if item == "getLogger":
			return self.getLogger
		elif item == "success":
			return self.success
		elif item == "SUCCESS":
			return self.SUCCESS
		# end if
			pass
		else:
			return getattr(_logging, item)

logging = _LoggingWrapper()


# -*- coding: utf-8 -*-
__author__ = 'luckydonald'

import sys
from ..dependencies import import_or_install
from ..logger import logging
logger = logging.getLogger(__name__)
import datetime as datetime
import_or_install("pytz", "pytz")
from pytz import timezone

__all__ = ["get_local_time", "loop_seconds_between_times"]


if sys.version < "3":
	def get_local_time(time, time_zone):
		return _get_local_time(time, time_zone)
	def loop_seconds_between_times(datetime1, datetime2):
		return _loop_seconds_between_times(datetime1, datetime2)
else:
	from .py3 import get_local_time, loop_seconds_between_times
#end if


def _get_local_time(time, time_zone):
	if time_zone is None:
		time_zone = "Europe/Berlin" # todo: config
	return timezone('UTC').localize(time).astimezone(timezone(time_zone))


def _loop_seconds_between_times(datetime1, datetime2):
	delta = datetime2 - datetime1
	seconds = delta.total_seconds()
	for x in range(0, int(seconds)):
		yield (datetime1 + datetime.timedelta(seconds=x))
	yield datetime2  # so loop_seconds_between_times(dat,dat) is at least called once with dat.




# -*- coding: utf-8 -*-
from random import randint

__author__ = 'luckydonald'

def get_headers():
	headers = {}
	headers["Server"] = ["iPod Touch, iOS 2.3.2", "Banana 0.2", "KONICHIWA/1.0", "'; DROP TABLE servertypes; -"][randint(0,3)]
	headers["X-Powered-By"] = ["Magical Ponies", "Rats in our Basement", "Unicorns", "Friendship", "TONS OF SUGAR", "coffee", "Bananas and Rum"][randint(0,6)]
	headers["X-Best-Pony"] = "Littlepip"
	headers["X-Answer"] = "42"
	headers["X-Never-Gonna"] = "Give you up."
	headers["X-Nananana"] = "Batman!"
	return headers


# -*- coding: utf-8 -*-
"""
NOT PY2 ready!
>>> from luckydonaldUtils.eastereggs.quotes import get_quote
>>> get_quote("de")
>>> get_quote()
"""

from random import randint
from ..dependencies import import_or_install
import_or_install("weakreflist", "weakreflist")
from weakreflist.weakreflist import WeakList  # pip install weakreflist

__author__ = 'luckydonald'


QUOTES = [
	"Littlepip is best pony!",
	{"en": "May contain nuts", "de": "Kann Spuren von Nüssen enthalten"},
	{"en": "0% sugar", "de": "0% Zucker"},
	{"en": "100% pure", "de":"100% rein"},
	{"en": "12345 is a bad password","de": "12345 ist ein schlechtes Passwort"},
	"90% bug free!",
	{"en": "Any computer is a laptop if you're brave enough", "de":"Jeder Computer ist ein Laptop, wenn du mutig bist"},
	"As seen on TV",
	"Ask your doctor",
	"Awesome",
	"20% cooler",
	"Muffins!",
	"Classy",
	"Do not distribute",
	"Don't look directly at the bugs",
	"Free dental included",
	"Gasp!",
	{"de": "Keuch!"},
	"Google anlyticsed!",
	{"en": "Günter was here", "de": "Günter war hier"},
	"Hot tamale, hot hot tamale!",
	"Now in 3D",
	"Coded in Python 3",
	{"en": "Lives in a pineapple under the sea", "de": "Wohnt in 'ner Ananas ganz tief im Meer"},
	"Limited edition!",
	"Keep calm and carry on",
	{"de": "Nicht nur sauber sondern rein"},
	{"de": "Mit 99% weniger Zucker", "en":"With 99% less sugar"},
	{"de": "Besser als herkömmliche Produkte"},
	{"en": "Supercalifragilisticexpialidocious", "de":"Superkalifragilisticexpialigetisch"},
	"War. War never changes.",
	{"en": "The sky is the limit", "de":"Alles ist möglich"},
	{"en": "Turing complete", "de": "Turing-vollständig"},
	"Ultimate edition",
	{"en": "Uninflammable", "de": "Nicht brennbar"},
	{"en": "Water proof", "de": "Wasserdicht"},
	{"en": "Vote for net neutrality!", "de": "Stimme für die Netzneutralität!"},
	{"de": "Unplattbar"},
	"What's up, Doc?",
	{"en": "You can't explain that", "de": "Das kannst du nicht erklären"},
	"Multithreaded",
	"0xffffff",
	{"en": "#c0ffee is not a brown color", "de": "Die Farbe #c0ffee ist nicht braun."},
	{"en": "Cooler than Spock", "de": "Cooler als Spock"},
	{"de": "Achten Sie auf die Goldkante, es lohnt sich."},
	{"de": "Afri-Cola überwindet den toten Punkt."},
	{"de": "Dahinter steckt immer ein kluger Kopf."},
	{"de": "Mach mal Pause..."},
	{"de": "Dieses Produkt ist Glutenfrei."},
	{"de": "Milch macht müde Männer munter"},
	{"de": "Zu Risiken und Nebenwirkungen fragen Sie ihren Arzt oder Apotheker"},
	{"de": "Jetzt mit mehr Vitaminen"},
	{"de": "Hier könnte Ihre Werbung stehen"},
	"Pon <3",
	{"en":"Pon3 is best musician"},
	{"en":"Batteries not included.", "de": "Batterien nicht enthalten"},
	{"de": "Abbildung ähnlich"},
	{"de": "Serviervorschlag"},
	{"en": "So kawaii"},
	"Aloha",
	{"de": "Anscheinend können Sie lesen", "en": "It seems you can read"},
	"DINKELBERG!",
	#{"de": ".sträwkcür tsi txeT reseiD", "en": ".desrever si txet sihT"},
	"The cake is a lie",
	#{"de": ""},
	#{"en": "", "de": ""},
]
QUOTES_BY_LANGUAGE = {"en": WeakList()}


""" Store them ordered. """
for quote in QUOTES:
	if isinstance(quote, dict):
		for language, text in quote.items():  # todo: python 2 iter_something()
			if not language in QUOTES_BY_LANGUAGE:
				QUOTES_BY_LANGUAGE[language] = WeakList()
			#end if
			QUOTES_BY_LANGUAGE[language].append(quote[language])
		#end for
	#end if
#end for
for quote in QUOTES:
	if isinstance(quote, str):
		for language in QUOTES_BY_LANGUAGE.keys():  # todo: py2:viewkeys():
			QUOTES_BY_LANGUAGE[language].append(quote)
		#end for
	#end if
#end for


def get_quote(language="en"):
	"""
	Get a random quote.

	:param language: default: "en"
	:return:
	"""
	qs = QUOTES_BY_LANGUAGE[language]
	return qs[randint(0, len(qs)-1)]
#end def



# -*- coding: utf-8 -*-

__author__ = 'luckydonald'

from .. import py3

if py3:
	def binary_ip_to_str(host):
		return ".".join([str(x) for x in host])
else:
	def binary_ip_to_str(host):
		return ".".join([str(ord(x)) for x in host])


# -*- coding: utf-8 -*-

from __future__ import division
__author__ = 'luckydonald'

from .dependencies import import_or_install
from .files import gettempdir
from .encoding import to_binary as b
import hashlib
import mimetypes
import sys
import os

DictObject = import_or_install("DictObject.DictObject", "DictObject")  # from DictObject import DictObject
magic = import_or_install("magic", "python-magic")  # import magic
requests = import_or_install("requests", "requests")  # import requests
import_or_install("progressbar", ("progressbar" if sys.version < "3.3" else "progressbar33"))
from progressbar import Widget
from progressbar import ETA, Percentage, Bar, FileTransferSpeed, ProgressBar
from requests.packages.urllib3.exceptions import HTTPError

import logging
logger = logging.getLogger(__name__)

HEADERS = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X10.69; rv:4458.42) Gecko/4458 Firefox/69.0 Pon3Downloader'}


def get_json(url, objectify=True, **kwargs):
	kwargs.setdefault("headers", HEADERS)
	json = requests.get(url, **kwargs).json()
	if objectify:
		return DictObject.objectify(json)
	return json


def download_file(url, used_cached=True, download_folder=None, temp_folder_name="luckydonald-utils", return_mime=False, return_buffer=False, progress_bar=False, **requests_kwargs):
	"""
	Downloads a file. Filename is the md5 hash of the url.

	:param url: The url you want to download.

	:param used_cached: If the url was already downloaded, that file will not be overwritten.
	The files name is just a hash of the url. So the same filename means same hash means same url.

	:param download_folder:
	Path to the download dir.
	If None, a folder will be created inside the /tmp/ folder equivalent of your system, with the name from `temp_folder_name`.

	:param temp_folder_name:
	Name of the folder to be created/used in the /tmp/ equivalent of your system.
	This is option ignored, if `download_folder` is given. 	Default: "luckydonald-utils"

	:param return_mime: If set to true, a (filename, mime) tuple is returned, else just the filename.

	:param return_buffer: If set to true, nothing will be written to disk,
	you'll get the buffer returned instead of the filename.

	:param progress_bar: If it should display a progress bar.

	:param **requests_kwargs: All other arguments are piped to the requests.get() call.

	:return: The path of the file. If `return_buffer` is true, instead it contains the buffer.
	If `return_mime` is true, it is a tuple of that and the mime, (file_path, mime) or (buffer, mime)

	"""
	requests_kwargs.setdefault("headers", HEADERS)
	bar = Bar(marker="#", left="[", right="]")
	if not return_buffer:
		if not download_folder:
			download_folder = gettempdir(temp_folder_name=temp_folder_name)
	#end if not return_buffer
	try:
		logger.debug("DL: Downloading from '{url}'.".format(url=url))
		if progress_bar:
			requests_kwargs["stream"] = True
			response = requests.get(url, **requests_kwargs)
			total_length = response.headers.get('content-length')
			if total_length is None: # no content length header
				widgets =["Downloading Song: ", bar, " ", DoneWidget(init="Connecting...", started="Unknown Size.", finished="Complete.")]
				pbar = ProgressBar(widgets=widgets).start()
				logger.debug("No content-length provided.")
				image_buffer = response.content
				pbar.finish()
			else:
				dl = 0
				image_buffer = b("")
				total_length = int(total_length)
				widgets =["Downloading Song: ",bar, " ", Percentage(), " ", FileTransferSpeed(), ", ", ETA()]
				pbar = ProgressBar(widgets=widgets, maxval=total_length).start()
				for data in response.iter_content(chunk_size=int(total_length/100)):
					dl += len(data)
					image_buffer += data
					pbar.update(dl)
				#end for
				pbar.finish()
				# now is downloaded.
				logger.debug("Download completed.")
		else:
			image_buffer = requests.get(url, **requests_kwargs).content
	except HTTPError as e:
		logger.exception("DL: Error in URL '" + url + "'.\n" + str(e))
		raise
		#return (None, None) if return_mime else None
	except Exception as e:
		logger.exception("DL: Error in Download '" + url + "'.\n" + str(e))
		raise
		#return (None, None) if return_mime else None
	mime = magic.from_buffer(image_buffer, mime=True).decode("utf-8")
	suffix = (mimetypes.guess_extension(mime) if mime else ".unknown") or ".unknown"
	if return_buffer:
		logger.debug("DL: Requested Buffer, not creating/checking dirs/files.")
		if return_mime:
			return image_buffer, mime
		return image_buffer
	else: # -> not return_buffer:
		file_name = str(hashlib.md5(url.encode()).hexdigest()) + suffix
		file_name = os.path.join(download_folder, file_name)
		if os.path.isfile(file_name):
			if used_cached:
				logger.debug("DL: File exists, using cached: %s" % file_name)
				if return_mime:
					return file_name, mime
				return file_name
			logger.debug("DL: File exists, redownloading: %s" % file_name)
		else:
			logger.debug("DL: File does not exist, downloading: %s" % file_name)
			if not os.path.exists(os.path.dirname(file_name)):
				logger.debug("DL: Download Folder does not exists. Creating.")
				os.makedirs(os.path.dirname(file_name))
		logger.debug("DL: Writing download from '%s' to file '%s'" % (url, file_name))
		try:
			with open(file_name, 'wb') as f:
				f.write(image_buffer)
		except Exception as e:
			logger.error("DL: Error in writing download to disk: '" + url + "' to '" + file_name + "'.\n" + str(e))
			file_name = None
		if return_mime:
			return file_name, mime
		return file_name
	#end if-else


class DoneWidget(Widget):
	"""
	Can display 3 values.
	"""
	def __init__(self, init="Not started.", started="Running.", finished="Done."):
		self.init_text = init
		self.started_text = started
		self.finished_text = finished

	def update(self, pbar):
		if pbar.start_time is None:
			return self.init_text
		elif not pbar.finished:
			return self.started_text
		else:
			return self.finished_text



# -*- coding: utf-8 -*-
__author__ = 'luckydonald'

from setuptools import setup
from luckydonaldUtils import VERSION
from luckydonaldUtils.dependencies import find_submodules


long_description = """A collection of utilities I use across different projects"""

# http://peterdowns.com/posts/first-time-with-pypi.html
# $ python setup.py register -r pypi
# $ python setup.py sdist upload -r pypi

main_package = 'luckydonaldUtils'

install_requires = ["pip", "setuptools"]  # both should be installed by default.
try:
	import importlib
except ImportError:
	install_requires.append('importlib')

setup(
	name="luckydonald-utils",
	packages=find_submodules(main_package),
	version=VERSION,
	author="luckydonald",
	author_email="code@luckydonald.de",
	description=long_description,
	license="GPLv3+",
	keywords="luckydonald utils utilities utility python dependencies dependency download progress bar encoding files interactions json update store text xml time network logger color images webserver django CSRF headers eastereggs",
	url="https://github.com/luckydonald/python-utils",
	install_requires=install_requires,
	long_description=long_description,
	classifiers=[
		"Development Status :: 4 - Beta",
		"Intended Audience :: Developers",
		"Topic :: Utilities",
		"License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
	],
)



# -*- coding: utf-8 -*-
"""
Tools to use with django's HttpRequests.
"""
__author__ = 'luckydonald'


from django.middleware.csrf import CsrfViewMiddleware

def check_csrf(request):
	"""
	This checks if the csrf check was passed.

	:param request: the request
	:type  request: django.http.HttpRequest

	:return: If it did pass the check.
	:rtype : bool
	"""
	reason = CsrfViewMiddleware().process_view(request, None, (), {})
	if reason:
		# CSRF failed
		return False
	return True
#end def

def GET_to_bool(request, key):
	"""
	Parses a GET parameter in the request as bool.
	"true" becomes True, "false" becomes False, "null" becomes None.

	:param request: the request
	:type  request: django.http.HttpRequest

	:param key: the key in request.GET
	:type  key: str

	:return: Boolean or None
	:rtype : bool | None
	"""
	clone_manual = request.GET[key]
	assert clone_manual in ["true", "false", "null"]
	clone_manual_bool = None
	if clone_manual == "true":
		clone_manual_bool = True
	if clone_manual == "false":
		clone_manual_bool = False
	return clone_manual_bool



# -*- coding: utf-8 -*-
# https://code.google.com/p/modwsgi/wiki/ReloadingSourceCode
"""
Add auto reloading capabilities.

Use in your wsgi file like this:

>>> import os
>>>
>>> from luckydonaldUtils.network.mod_wsgi import reloader
>>> reloader.start(interval=1.0)
>>> reloader.track(os.path.join(os.path.dirname(__file__), 'site.cf'))
>>>
>>> def application(environ, start_response):
>>>    ...

"""

from __future__ import print_function
__author__ = 'luckydonald'


from luckydonaldUtils.logger import logging  # pip install luckydonald-utils
logger = logging.getLogger(__name__)


import os
import sys
import time
import signal
import threading
import atexit
try:
	from queue import Queue  # py3
except ImportError:
	from Queue import Queue  # py2
#end try

_interval = 1.0
_times = {}
_files = []

_running = False
_queue = Queue()
_lock = threading.Lock()

def warning(*objs):
	print(*objs, file=sys.stderr)

def _restart(path):
	_queue.put(True)
	prefix = 'monitor (pid=%d):' % os.getpid()
	warning('%s Change detected to \'%s\'.' % (prefix, path))
	warning('%s Triggering process restart.' % prefix)
	os.kill(os.getpid(), signal.SIGINT)

def _modified(path):
	try:
		# If path doesn't denote a file and were previously
		# tracking it, then it has been removed or the file type
		# has changed so force a restart. If not previously
		# tracking the file then we can ignore it as probably
		# pseudo reference such as when file extracted from a
		# collection of modules contained in a zip file.

		if not os.path.isfile(path):
			return path in _times

		# Check for when file last modified.

		mtime = os.stat(path).st_mtime
		if path not in _times:
			_times[path] = mtime

		# Force restart when modification time has changed, even
		# if time now older, as that could indicate older file
		# has been restored.

		if mtime != _times[path]:
			return True
	except:
		# If any exception occured, likely that file has been
		# been removed just before stat(), so force a restart.

		return True

	return False

def _monitor():
	while 1:
		# Check modification times on all files in sys.modules.

		for module in sys.modules.values():
			if not hasattr(module, '__file__'):
				continue
			path = getattr(module, '__file__')
			if not path:
				continue
			if os.path.splitext(path)[1] in ['.pyc', '.pyo', '.pyd']:
				path = path[:-1]
			if _modified(path):
				return _restart(path)

		# Check modification times on files which have
		# specifically been registered for monitoring.

		for path in _files:
			if _modified(path):
				return _restart(path)

		# Go to sleep for specified interval.

		try:
			return _queue.get(timeout=_interval)
		except:
			pass

_thread = threading.Thread(target=_monitor)
_thread.setDaemon(True)

def _exiting():
	try:
		_queue.put(True)
	except:
		pass
	_thread.join()

atexit.register(_exiting)

def track(path):
	if not path in _files:
		_files.append(path)

def start(interval=1.0):
	global _interval
	if interval < _interval:
		_interval = interval

	global _running
	_lock.acquire()
	if not _running:
		prefix = 'monitor (pid=%d):' % os.getpid()
		warning(sys.stderr, '%s Starting change monitor.' % prefix)
		_running = True
		_thread.start()
	_lock.release()